#include "wiced.h"
#include "../simpleCli.h"
#include "SPITest.h"


//HDC1050 Slave
#define N25Q032A_CMD_READ_ID        0x9F// 0x9E/0x9F
#define N25Q032A_ID					0x20BA1610
//HDC1050 Offset
#define N25Q032A_MANUFACTURER_ID    0x20//	Byte 1 Manufacturer ID

#define SPI_MAX_DEVICES 3

wiced_spi_device_t *_spi_dev=&wiced_spi_flash;

static int gBitRate[SPI_MAX_DEVICES] = {0};
static unsigned char gOffsetAddress[SPI_MAX_DEVICES] = {N25Q032A_CMD_READ_ID,N25Q032A_CMD_READ_ID,0x00};


static int BitRate(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len, gBitRate[index] == 0 ? " (1MHz)" : " (100KHz)");
		return 0;
	}
	console_printf("\r\n*) Please input the bitrate ( 0: 1MHz , 1: 100KHz ): \r\n");
	gBitRate[index] = GetInt_wait();
    
    if(gBitRate[index] < 0 || gBitRate[index] > 1) gBitRate[index] = 0;
	return 0;
}

static int OffsetAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gOffsetAddress[index]);
		return 0;
	}
	console_printf("\r\n*) Please input the offset address ( 0x00 ~ 0xFF ): \r\n");
	gOffsetAddress[index] = (char)(GetInt_wait()&0xFF);
    
    if(gOffsetAddress[index] < 0 || gOffsetAddress[index] > 0xFF) gOffsetAddress[index] = N25Q032A_CMD_READ_ID;
	return 0;
}

static int Read(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	unsigned char tx[5]={N25Q032A_CMD_READ_ID, 0, 0, 0, 0};
	unsigned char rx[5] = {0, 0, 0, 0, 0};
    wiced_spi_message_segment_t spi_msg;

    switch(gBitRate[index]) {
        case 0:
        	_spi_dev->speed=1000000;
        break;

        case 1:
        	_spi_dev->speed=100000;
        break;

        default:
        	_spi_dev->speed=1000000;
        break;
    }
	wiced_spi_init(_spi_dev);

    spi_msg.tx_buffer=tx;
    spi_msg.rx_buffer=rx;
    spi_msg.length=sizeof(tx);
    wiced_spi_transfer( _spi_dev, &spi_msg, 1 );

	console_printf("\r\n");
	console_printf("Result: 0x%2X 0x%2X 0x%2X 0x%2X\n",rx[1],rx[2],rx[3],rx[4]);
	console_printf("\r\n");

	return true;
}

int SPITest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    if(index < 0 || index >= SPI_MAX_DEVICES) return 0;
    
	simple_cmds cmds[] = {
            { "SPI Test" /*title*/,  					0,NULL },
            { "BitRate",							    index,BitRate },
            { "Offset Address",							index,OffsetAddress },
            { "Read",			                        index,Read },
            { NULL,										0,NULL }
    };

	return simple_cli_list(cmds);
}

int SPI_Auto_Test(void)
{
	wiced_result_t result;
	unsigned char tx[5]={N25Q032A_CMD_READ_ID, 0, 0, 0, 0};
	unsigned char rx[5] = {0, 0, 0, 0, 0};
    wiced_spi_message_segment_t spi_msg;

    _spi_dev->speed=1000000;
    result=wiced_spi_init(_spi_dev);
    if(result != WICED_SUCCESS){
    	console_printf("SPI_err1 ===> ");
    	wiced_spi_deinit(_spi_dev);
    	return result;
    }

    spi_msg.tx_buffer=tx;
    spi_msg.rx_buffer=rx;
    spi_msg.length=sizeof(tx);
    result=wiced_spi_transfer( _spi_dev, &spi_msg, 1 );
    if(result != WICED_SUCCESS){
    	console_printf("SPI_err2 ===> ");
    	wiced_spi_deinit(_spi_dev);
    	return result;
    }
    wiced_spi_deinit(_spi_dev);

    if((rx[1]<<24 | rx[2]<<16 | rx[3]<<8 | rx[4])==N25Q032A_ID)
    	return WICED_SUCCESS;
    else{
    	console_printf("N25Q032A_ID=%d ===>", (rx[1]<<24 | rx[2]<<16 | rx[3]<<8 | rx[4]));
    	return -1;
    }
}
