#include "wiced.h"
#include "../simpleCli.h"
#include "Susi4.h"
#include "susiI2CTest.h"
#include "susi_i2c.h"

//HDC1050 Slave
#define TI_HDC1050_DEVICE_ADDR      0x40
#define NXP_PCA9545APW_DEV_ADDR     0x70
#define I2C_ID                      0


//HDC1050 Offset
#define TI_HDC1050_MANUFACTURER_ID_ADDR	0xFE
#define TI_HDC1050_MANUFACTURER_ID		0x5449

static int gBitRate = 0;
static unsigned char gSlaveAddress = TI_HDC1050_DEVICE_ADDR;
static unsigned char gOffsetAddress = TI_HDC1050_MANUFACTURER_ID_ADDR;

static int BitRate(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (%dKHz)",gBitRate == 0 ? 100 : 400);
		return 0;
	}
	console_printf("\r\n*) Please input the bitrate ( 0: 100KHz, 1: 400KHz ): \r\n");
	gBitRate = GetInt_wait();
    
    if(gBitRate==1){
        SusiI2CSetFrequency(I2C_ID, 400);
    }
    else{
        SusiI2CSetFrequency(I2C_ID, 100);
        gBitRate = 0;
    }
	return 0;
}

static int SlaveAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gSlaveAddress);
		return 0;
	}
	console_printf("\r\n*) Please input the slave address ( 0x00 ~ 0x7F ): \r\n");
	gSlaveAddress = (unsigned char)(GetInt_wait()&0xFF);
	return 0;
}


static int OffsetAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gOffsetAddress);
		return 0;
	}
	console_printf("\r\n*) Please input the offset address ( 0x00 ~ 0xFF ): \r\n");
	gOffsetAddress = (unsigned char)(GetInt_wait()&0xFF);
	return 0;
}

static int Read_HDC105_ID(int index, char *string, int len, void *printer) {

	uint8_t val[2]={0};
	SusiStatus_t result;

	if(string != NULL && len != 0)
		return 0;

	console_printf("\r\n");
	result=SusiI2CReadTransfer(I2C_ID, gSlaveAddress, TI_HDC1050_MANUFACTURER_ID_ADDR, val, sizeof(val));

    if(result == SUSI_STATUS_SUCCESS){
    	console_printf("Read from 0x%02X--0x%02X\r\n",gSlaveAddress, gOffsetAddress);
    	console_printf("Susi I2C Result: 0x%02X%02X\r\n",val[0],val[1]);
    	console_printf("\r\n");
    }
    else{
    	console_printf("Read_HDC105_ID error.\r\n");
    }

    wiced_rtos_delay_microseconds(30);
	return true;
}

int I2CTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    
	simple_cmds cmds[] = {
            { "I2C Test" /*title*/,  					    0, NULL },
            { "BitRate",							    index, BitRate },
            { "Slave Address",							index, SlaveAddress },
            { "Offset Address",							index, OffsetAddress },
            { "Read TI_HDC105 ID",			        	index, Read_HDC105_ID },
			//{ "I2C-SW Channel Select",			        index, I2C_SW_Sel },
			//{ "Set I2C-SW Channel",				        index, Set_I2C_SW },
            //{ "EEP_24C32_Write",			            index, EEP_24C32_Write },
            //{ "EEP_24C32_Read",			                index, EEP_24C32_Read },
            { NULL,										0, NULL }
    };
	return simple_cli_list(cmds);
}

int I2C_Auto_Test(void)
{
    uint8_t val[2]={0};
    SusiStatus_t result;

    result=SusiI2CReadTransfer(I2C_ID, gSlaveAddress, TI_HDC1050_MANUFACTURER_ID_ADDR, val, sizeof(val));

    if(result == SUSI_STATUS_SUCCESS)
        return result;

    if((val[0]<<8 | val[1])==TI_HDC1050_MANUFACTURER_ID)
    	return WICED_SUCCESS;
    else{
    	console_printf("MANUFACTURER_ID=%d ===>", (val[0]<<8 | val[1]));
    	return -1;
    }
}



